//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2005 Xilinx, Inc.
//     All rights reserved.
//

module flash_cntl
  (
   clk,
   reset,
   start_address,
   size,
   start_reading,
   increment_address,
   reading_flash,
   sram_flash_oe_n,
   flash_audio_reset_n,
   flash_ce2,
   flash_address
   );

   input            clk;
   input            reset;
   input [25:0]     start_address;
   input [25:0]     size;
   input            start_reading;
   input            increment_address;
   output           reading_flash;
   output           sram_flash_oe_n;
   output           flash_audio_reset_n;
   output           flash_ce2;
   output [25:0]    flash_address;
   
   parameter [1:0]  C_RESET_STATE = 2'd0,
                    C_IDLE_STATE = 2'd1,
                    C_READ_STATE = 2'd2;

   reg [25:0]       address_register;
   reg [1:0]        cstate, nstate;
   reg [2:0]        reset_timeout;
   
   // Address is incremented by 4 bytes since data is 32 bits
   always @(posedge clk)
     if (cstate == C_RESET_STATE)
       address_register <= start_address;
     else if (increment_address && (address_register != (start_address + size - 4)))
//     else if (increment_address)
       address_register <= address_register + 4;

   // Spec:  After RP# is negated, it's 100ns before flash data is available
   // Over-engineer by assuming a 10ns clock period.  So, delay for at least
   // 10 clock cycles.
   always @(posedge clk)
     if (cstate == C_RESET_STATE)
       reset_timeout <= ~0;
     else if (reset_timeout != 0)
       reset_timeout <= reset_timeout - 1;

   always @(posedge clk or posedge reset)
     if (reset)
       cstate <= C_RESET_STATE;
     else
       cstate <= nstate;

   always @(cstate or start_reading or reset_timeout)
     begin
        nstate = cstate;
        case (cstate)
          C_RESET_STATE:
            if (start_reading)
              nstate = C_IDLE_STATE;
          C_IDLE_STATE:
            if (reset_timeout == 0)
              nstate = C_READ_STATE;
          C_READ_STATE:
            ;
          default:
            // This case should never happen
            ;
        endcase
     end
   
   assign reading_flash = (cstate == C_READ_STATE);
   
   // Should be ok to tie to constant.
   assign           sram_flash_oe_n = 0;

   /*
    * RP# spec: Low 35 us if during a BLOCK ERASE, PROGRAM, or
    *           LOCK BIT CONFIGURATION operation (100 ns in that
    *           case)
    * It doesn't appear necessary to assert at power-up, but we'll
    * do it anyway.
    * 
    * If at any time, the FPGA's PROG_B is asserted, cstate will
    * be set to C_RESET_STATE and RP# is held throughout the FPGA
    * housecleaning which hopefully is longer than 35 us in case
    * a flash operation was interrupted.
    * At power-up, reset is asserted, so RP# is asserted again
    * until FPGA housecleaning is completed.
    */ 
   assign           flash_audio_reset_n = ~(cstate == C_RESET_STATE);

   // Set to 1 to activate Flash components
   assign           flash_ce2 = 1;

   assign           flash_address = address_register;
   
endmodule
