//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2005 Xilinx, Inc.
//     All rights reserved.
//

module flash_data
  (
   clk,
   capture,
   shift,
   byte_mode,
   flash_data,
   last_shift,
   data_register
   );

   input clk;
   input capture;
   input shift;
   input byte_mode;
   input [31:0] flash_data;
   output       last_shift;
   output [31:0] data_register;

   reg           sentry;
   reg [31:0]    data_register;
   reg           last_shift;

   // synthesis translate_off
   initial data_register = 0;
   initial sentry = 0;
   // synthesis translate_on
   
   // Shift left by 1 bit or 8 bits depending on mode
   always @(posedge clk)
     if (capture)
       { data_register, sentry } <= { flash_data, 1'b1 };
     else if (shift)
       if (byte_mode)
         { data_register, sentry } <= { data_register[23:0], sentry, 8'b0 };
       else
         { data_register, sentry } <= { data_register[30:0], sentry, 1'b0 };
         
   always @(data_register or sentry or byte_mode)
     begin
        case (byte_mode)
          0:
            begin
               // sentry '1' is at bit position 30
               last_shift = { data_register[31-1-1:0], sentry } == 0;
            end
          1:
            begin
               // sentry '1' is at bit position 23
               last_shift = { data_register[31-8-1:0], sentry } == 0;
            end
        endcase
     end
   
endmodule
