//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2005 Xilinx, Inc.
//     All rights reserved.
// 

module fpga_cntl
  (
   reset,
   clk,
   fpga_done,
   fpga_init,
   fpga_m2,
   fpga_m1,
   fpga_m0,
   cfgmode_serial,
   cfgmode_selectmap,
   cfgmode_master,
   cfgmode_other,
   fpga_cfg_housecleaning,
   fpga_cfg_program,
   fpga_cfg_startup,
   fpga_cfg_complete,
   fpga_cfg_error
   );

   input reset;
   input clk;
   input fpga_done;
   input fpga_init;
   input fpga_m2;
   input fpga_m1;
   input fpga_m0;

   output cfgmode_serial;
   output cfgmode_selectmap;
   output cfgmode_master;
   output cfgmode_other;
   output fpga_cfg_housecleaning;
   output fpga_cfg_program;
   output fpga_cfg_startup;
   output fpga_cfg_complete;
   output fpga_cfg_error;

   // Must use grey encoding of state machine since
   // the state machine inputs are asynchronous to the state machine
   // clock
   parameter [1:0] C_IDLE_STATE=2'd0,
                   C_PROGRAM_STATE=2'd1,
                   C_SUCCESS_STATE=2'd3,
                   C_ERROR_STATE=2'd2;
   
   wire            cfgmode_slave_serial;
   wire            cfgmode_master_serial;
   wire            cfgmode_slave_selectmap;
   wire            cfgmode_master_selectmap;
   reg [1:0]       cstate, nstate;
   reg [7:0]       cnt;
   reg [10:0]      init_counter;
   wire [2:0]      cfgmode = { fpga_m2, fpga_m1, fpga_m0 };
   reg             fpga_init_1d, fpga_init_2d;
   reg             fpga_init_rise_detected;
   
   assign     cfgmode_slave_serial = (cfgmode == 3'b111);
   assign     cfgmode_master_serial = (cfgmode == 3'b000);
   assign     cfgmode_slave_selectmap = (cfgmode == 3'b110);
   assign     cfgmode_master_selectmap = (cfgmode == 3'b011);

   assign     cfgmode_serial = cfgmode_slave_serial | cfgmode_master_serial;
   assign     cfgmode_selectmap = cfgmode_slave_selectmap | cfgmode_master_selectmap;
   assign     cfgmode_master = cfgmode_master_serial | cfgmode_master_selectmap;
   assign     cfgmode_other =
              ~(cfgmode_slave_serial | cfgmode_master_serial
                | cfgmode_slave_selectmap | cfgmode_master_selectmap);

   always @(posedge clk)
     if (reset)
       cstate <= C_IDLE_STATE;
     else
       cstate <= nstate;

   // On power-up or after a FPGA reset
   //   1. Wait for FPGA INIT pin to transition from low to high.
   //      On power-up, it is assumed that the CPLD will be functional prior
   //      to this event.
   //   2. Wait some time afterwards before starting configuration.
   always @(posedge clk)
     begin
        fpga_init_1d <= fpga_init;
        fpga_init_2d <= fpga_init_1d;
     end

   always @(posedge clk)
     if (reset)
       fpga_init_rise_detected <= 0;
     else
       fpga_init_rise_detected <= fpga_init_rise_detected ||
                                  (fpga_init_1d & ~fpga_init_2d);
   
   always @(posedge clk)
     if (reset)
       init_counter <= 0;
     else if (fpga_init_rise_detected)
       init_counter <= init_counter + 1;

   always @(cstate or init_counter or fpga_init or fpga_done)
     begin
        nstate = cstate;
        case (cstate)
          C_IDLE_STATE:
            if (init_counter[10])
              nstate = C_PROGRAM_STATE;
          C_PROGRAM_STATE:
            if (fpga_done)
              nstate = C_SUCCESS_STATE;
            else if (~fpga_init)
              nstate = C_ERROR_STATE;
          C_SUCCESS_STATE:
            ;
          C_ERROR_STATE:
            ;
          default:
            ;
        endcase
     end

   always @(posedge clk)
     if (reset)
       cnt <= ~0;
     else if ((cstate == C_SUCCESS_STATE) && (cnt != 0))
       cnt <= cnt - 1;

   // controller state
   assign fpga_cfg_housecleaning = (cstate == C_IDLE_STATE);
   assign fpga_cfg_program = (cstate == C_PROGRAM_STATE);
   assign fpga_cfg_startup = (cstate == C_SUCCESS_STATE) && (cnt != 0);
   assign fpga_cfg_complete = (cstate == C_SUCCESS_STATE) && (cnt == 0);
   assign fpga_cfg_error = (cstate == C_ERROR_STATE);
   
endmodule
