//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2005 Xilinx, Inc.
//     All rights reserved.
//

module fpga_slave_serial
  (
   clk,
   reset,
   enable,
   send_data,
   data_register,
   get_next_bit,
   cclk,
   din
   );

   input clk;
   input reset;
   input enable;
   input send_data;
   input [31:0] data_register;
   output get_next_bit;
   output cclk;
   output din;

   reg    cclk;
   wire   din;

   parameter [1:0] IDLE_STATE = 2'd0,
                   CLOCK1_STATE = 2'd1,
                   CLOCK0_STATE = 2'd2;
   
   reg [1:0] cstate, nstate;
   reg       ncclk;
   reg       get_next_bit;
   
   always @(cstate or send_data)
     begin
        nstate = IDLE_STATE;
        ncclk = 0;
        case (cstate)
          IDLE_STATE:
            begin
               if (send_data)
                 begin
                    nstate = CLOCK1_STATE;
                    ncclk = 1;
                 end
            end
          CLOCK1_STATE:
            begin
               nstate = CLOCK0_STATE;
            end
          CLOCK0_STATE:
            begin
               if (send_data)
                 begin
                    nstate = CLOCK1_STATE;
                    ncclk = 1;
                 end
               else
                 nstate = IDLE_STATE;
            end
        endcase
     end
            
   always @(posedge clk)
     if (reset || ~enable)
       cstate <= IDLE_STATE;
     else
       cstate <= nstate;

   assign din = data_register[31];

   always @(posedge clk)
     cclk <= ncclk;

   always @(posedge clk)
     get_next_bit <= ncclk;
   
endmodule
