//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2005 Xilinx, Inc.
//     All rights reserved.
//

module reset_logic
  (
   clk,
   cpld_prog,
   fpga_prog_b,
   fpga_done,
   fpga_housecleaning,
   reset,
   global_disable
   );

   input clk;  // Must be a free-running clock
   input cpld_prog;
   input fpga_prog_b;
   input fpga_done;
   input fpga_housecleaning;
   
   output reset;
   output global_disable;

   reg    global_disable;
   reg    reset_n_ff;
   wire   reset_sig;
   
   /*
    * Have a global disable signal.
    * The only tricky case is if the CPLD is selected after
    * the FPGA is already configured.  In that case, we keep
    * the CPLD disabled until a PROG pulse to avoid contention
    * with the FPGA.
    */
   
   // synthesis translate_off
   initial global_disable = 0;
   // synthesis translate_on
   always @(fpga_prog_b or cpld_prog or fpga_done or fpga_housecleaning or fpga_done)
     if (!cpld_prog
         || fpga_housecleaning
         || fpga_done)
       global_disable <= 1;
     else
       if (!fpga_prog_b
           || (cpld_prog && ~fpga_done))
         global_disable <= 0;

   assign reset_sig = ~fpga_prog_b || !cpld_prog;

   /*
    * Asynchronous assertion of reset, synchronous negation
    */
   
   // synthesis translate_off
   initial reset_n_ff = 0;
   // synthesis translate_on
   always @(posedge clk or posedge reset_sig)
     if (reset_sig)
       reset_n_ff <= 0;
     else
       reset_n_ff <= 1;

   assign reset = ~reset_n_ff;
   
endmodule
