//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2005 Xilinx, Inc.
//     All rights reserved.
//

module system
  (
   cpld_prog,
   sysace_cfgaddr,
   sysace_clk,
   sysclk_100mhz,
   sram_flash_d,
   sram_flash_a,
   sram_flash_oe_n,
   flash_audio_reset_n,
   flash_ce2,
   fpga_dout_busy,
   fpga_din,
   fpga_done,
   fpga_init,
   fpga_prog_b,
   fpga_cs_b,
   fpga_m2,
   fpga_m1,
   fpga_m0,
   platflash_cpld_cclk,
   cpld_io
   );

   // Defines C_BITSTREAM_SIZE
   `include "system_target.v"

   input cpld_prog;
   input [2:0] sysace_cfgaddr;
   input       sysace_clk;
   input       sysclk_100mhz;

   // Flash
   inout [31:0] sram_flash_d;
   inout [23:0] sram_flash_a;
   inout        sram_flash_oe_n;
   inout        flash_audio_reset_n;
   inout        flash_ce2;

   // FPGA
   input        fpga_dout_busy;
   input        fpga_done;
   input        fpga_init;
   input        fpga_prog_b;
   inout        fpga_din;
   inout        fpga_cs_b;
   input        fpga_m2;
   input        fpga_m1;
   input        fpga_m0;
   inout        platflash_cpld_cclk;

   // General Purpose I/Os
   inout [6:1]  cpld_io;

   wire         platflash_cpld_I, platflash_cpld_O;
   wire         sram_flash_oe_n_O;
   wire         flash_audio_reset_n_O;
   wire         flash_ce2_O;
   wire         reset;
   wire         global_disable;
   wire         cfgmode_serial;
   wire         cfgmode_selectmap;
   wire         cfgmode_master;
   wire         cfgmode_other;
   wire         fpga_cfg_housecleaning;
   wire         fpga_cfg_program;
   wire         fpga_cfg_startup;
   wire         fpga_cfg_complete;
   wire         fpga_cfg_error;

   wire         start_reading;
   wire [25:0]  start_address;
   wire [25:0]  size;
   wire         increment_address;
   wire         reading_flash;
   wire [23:0]  sram_flash_a_O;
   wire [25:0]  flash_address;
   
   wire         flash_data_capture;
   wire         flash_data_shift;
   wire         flash_last_shift;
   wire [31:0]  flash_data_register;

   // Slave Serial signals
   wire         ss_next;
   wire         ss_cclk;
   wire         ss_din;

   wire [31:0]  sram_flash_d_O;
   wire [31:0]  sram_flash_d_I;
   
   clocks iclocks
     (
      .sysace_clk             (sysace_clk),
      .sysclk_100mhz          (sysclk_100mhz),
      .cpld_clk               (clk)
      );
   
   reset_logic ireset_logic
     (
      .clk              (clk),
      .cpld_prog        (cpld_prog),
      .fpga_prog_b      (fpga_prog_b),
      .fpga_done        (fpga_done),
      .fpga_housecleaning (~fpga_init),
      .reset            (reset),
      .global_disable   (global_disable)
      );

   fpga_cntl ifpga_cntl
     (
      .reset              (reset),
      .clk                (clk),
      .fpga_done          (fpga_done),
      .fpga_init          (fpga_init),
      .fpga_m2            (fpga_m2),
      .fpga_m1            (fpga_m1),
      .fpga_m0            (fpga_m0),
      .cfgmode_serial     (cfgmode_serial),
      .cfgmode_selectmap  (cfgmode_selectmap),
      .cfgmode_master     (cfgmode_master),
      .cfgmode_other      (cfgmode_other),
      .fpga_cfg_housecleaning (fpga_cfg_housecleaning),
      .fpga_cfg_program   (fpga_cfg_program),
      .fpga_cfg_startup   (fpga_cfg_startup),
      .fpga_cfg_complete  (fpga_cfg_complete),
      .fpga_cfg_error     (fpga_cfg_error)
      );

   generate
      if (C_BITSTREAM_SIZE <= 32'h0_100000)
        begin : size_1meg
           assign         size = 26'h0_100000;
           assign         start_address = {26{1'b1}} - (size * (8 - sysace_cfgaddr)) + 1;
        end
      else if (C_BITSTREAM_SIZE <= 32'h0_200000)
        begin : size_2meg
           assign         size = 26'h0_200000;
           assign         start_address = {26{1'b1}} - (size * (4 - sysace_cfgaddr[1:0])) + 1;
        end
   endgenerate
   
   assign       start_reading = fpga_cfg_program;
   assign       increment_address = fpga_cfg_program && reading_flash && flash_data_capture;
   
   flash_cntl iflash_cntl
     (
      .clk                   (clk),
      .reset                 (reset),
      .start_address         (start_address),
      .size                  (size),
      .start_reading         (start_reading),
      .increment_address     (increment_address),
      .reading_flash         (reading_flash),
      .sram_flash_oe_n       (sram_flash_oe_n_O),
      .flash_audio_reset_n   (flash_audio_reset_n_O),
      .flash_ce2             (flash_ce2_O),
      .flash_address         (flash_address)
      );

   assign       flash_data_capture = ~reading_flash || flash_last_shift && ss_next;
   assign       flash_data_shift = ss_next;
   
   flash_data iflash_data
     (
      .clk                   (clk),
      .capture               (flash_data_capture),
      .shift                 (flash_data_shift),
      .byte_mode             (cfgmode_selectmap),
      .flash_data            (sram_flash_d_I),
      .last_shift            (flash_last_shift),
      .data_register         (flash_data_register)
      );

   fpga_slave_serial ifpga_slave_serial
     (
      .clk                   (clk),
      .reset                 (reset),
      .enable                (cfgmode_serial && !cfgmode_master),
      .send_data             (reading_flash),
      .data_register         (flash_data_register),
      .get_next_bit          (ss_next),
      .cclk                  (ss_cclk),
      .din                   (ss_din)
      );
   
   // Flash
   assign       sram_flash_d_I[31:0] = sram_flash_d[31:0];
//   assign sram_flash_d[31:0] = 32'bz;

   // Use flash_address[25:2] since there are two flash chips providing data
   assign sram_flash_a[23:0] = (global_disable) ? 'bz : flash_address[25:2];
   assign sram_flash_oe_n = (global_disable) ? 1'bz : sram_flash_oe_n_O;  // Disable
   assign flash_audio_reset_n = (global_disable) ? 1'bz : flash_audio_reset_n_O;
   assign flash_ce2 = (global_disable) ? 1'bz : flash_ce2_O;  // Disable

   // FPGA
   assign fpga_din = (global_disable) ? 1'bz : ss_din;
   assign fpga_cs_b = (global_disable || !cfgmode_selectmap) ? 1'bz : 0;

   assign platflash_cpld_cclk =
          (cpld_prog && !cfgmode_master && !fpga_cfg_complete) ?
          ss_cclk : 1'bz;  // may have contention if cpld_prog set to 1 when FPGA is already configured and FPGA is driving CCLK
//          (global_disable || cfgmode_master) ? 1'bz : platflash_cpld_cclk_O;


   assign cpld_io[6] = 1'bz;
   assign cpld_io[5] = 1'bz;
   assign cpld_io[4] = (global_disable) ? 1'bz : sysace_clk;
   assign cpld_io[3] = 1'bz;
   assign cpld_io[2] = 1'bz;
   assign cpld_io[1] = 1'bz;

endmodule
